from typing import Any, Callable, Dict, List, Optional, Tuple
import json

from NotionPy import constants

Vector1 = List[Tuple[str, str, Optional[Any]]]
Vector2 = List[Tuple[str, str]]


def parse_for_pg_creation(
    db_id: str,
    data: Vector1,
    icon: Optional[str] = None,
    cover_url: Optional[str] = None,
) -> Dict:
    """
    function for collecting the inputed params into a notion acceptable dict for further conversion into json
    :param db_id:
        the database id in which the page will be created
    :param data:
        list of tuples each containing (property name , property type , value of property )
    :param icon:
        icon of page
    :param cover_url:
        url of the page's cover
    """
    data_dict = {}
    temp_dict = {}
    if icon is not None:
        temp_dict.update(constants.ICON(icon))
    if cover_url is not None:
        temp_dict.update(constants.COVER(cover_url))
    if data is not None:
        for name, type_, content in data:
            if type_.upper() not in {
                i for i in constants.__dict__.keys()
            }:  # check if property type exists
                raise ValueError(f"Property {type_} Doesn't exist")

            else:
                type_attr = getattr(constants, type_.upper())
                data_dict.update(type_attr(name, content))
        parsed_dict = constants.CREATING_PAGE_TEMPLATE(db_id, data_dict)
        parsed_dict.update(temp_dict)
        return parsed_dict


def parse_into_json(
    parsed_data: Callable[[Optional[Vector1]], Dict], indent: Optional[int] = None
):
    return json.dumps(parsed_data, indent=indent)


def parse_into_dict(json_data):
    return json.loads(json_data)


def parse_for_db_creation(
    pg_id: str,
    data: Vector2,
    title: Optional[str] = None,
    icon: Optional[str] = None,
    cover_url: Optional[str] = None,
) -> Dict:
    """
    function for collecting the inputed params into a notion acceptable dict for further conversion into json
    :param pg_id:
        the page id in which the database will be created as a subpage
    :param title:
        the title of the database
    :param data:
        list of tuples each containing ( property name , property type )
    :param icon:
        icon of database
    :param cover_url:
        url of the database's cover
    """
    data_dict = {}
    temp_dict = {}
    if icon is not None:
        temp_dict.update(constants.ICON(icon))
    if cover_url is not None:
        temp_dict.update(constants.COVER(cover_url))
    if data is not None:
        for name, type_ in data:
            if type_.upper() not in {
                i for i in constants.__dict__.keys()
            }:  # check if property type exists
                raise ValueError(f"Property {type_} Doesn't exist")

            else:
                type_attr = getattr(constants, type_.upper())
                type_attr_ = type_attr(name, None)
                type_attr_[name] = {type_: {}}

                data_dict.update(type_attr_)
        parsed_dict = constants.CREATING_DATABASE_TEMPLATE(pg_id, title, data_dict)
        parsed_dict.update(temp_dict)
        return parsed_dict


# not currently in use but for future updates
def parse_for_updating_db(
    db_id: str,
    data: Vector1,
    title: Optional[str] = None,
    icon: Optional[str] = None,
    cover_url: Optional[str] = None,
) -> Dict:
    data_dict = {}
    temp_dict = {}
    if icon is not None:
        temp_dict.update(constants.ICON(icon))
    if cover_url is not None:
        temp_dict.update(constants.COVER(cover_url))
    if data is not None:
        for name, type_, content in data:
            if type_.upper() not in {
                i for i in constants.__dict__.keys()
            }:  # check if property type exists
                raise ValueError(f"Property {type_} Doesn't exist")

            else:
                type_attr = getattr(constants, type_.upper())
                data_dict.update(type_attr(name, content))
        parsed_dict = constants.UPDATE_DB_TEMPLATE(db_id, data_dict)
        if title is not None:
            parsed_dict.update(constants.DB_TITLE(title))

        parsed_dict.update(temp_dict)
        return parsed_dict



from typing import List, Optional, Tuple, Any

import requests

from NotionPy import constants
from NotionPy.utils import parse_for_pg_creation, parse_into_json, parse_for_db_creation

Vector = List[Tuple[str, str, Optional[Any]]]


class Create:
    """
    class that creates pages or databases with provided data , icon and cover
    """

    TOKEN = None

    def __init__(self) -> None:
        pass

    def page(
        self,
        db_id: str,
        icon: Optional[str] = None,
        cover_url: Optional[str] = None,
        data: Optional[Vector] = None,
    ) -> None:
        data_ = parse_for_pg_creation(
            db_id=db_id, data=data, icon=icon, cover_url=cover_url
        )
        data_ = parse_into_json(data_)
        requests.post(
            url=constants.CREATE_PAGE_URL,
            headers=constants.HEADERS(Create.TOKEN),
            data=data_,
        )

    def db(
        self,
        page_id: str,
        title: str,
        icon: Optional[str] = None,
        cover_url: Optional[str] = None,
        data: Vector = None,
    ) -> None:
        data_ = parse_for_db_creation(
            pg_id=page_id, title=title, data=data, icon=icon, cover_url=cover_url
        )
        data_ = parse_into_json(data_)
        requests.post(
            url=constants.CREATE_DB_URL,
            headers=constants.HEADERS(Create.TOKEN),
            data=data_,
        )



from typing import List, Tuple, Optional, Any

import requests

from NotionPy import constants
from NotionPy.utils import parse_for_pg_creation, parse_into_json, parse_for_updating_db

Vector = List[Tuple[str, str, Optional[Any]]]


class Update:
    """
    class that updates the contents of (currently) pages
    """

    TOKEN = None

    def __init__(self) -> None:
        pass

    def page(
        self,
        pg_id: str,
        icon: Optional[str] = None,
        cover_url: Optional[str] = None,
        data: Optional[Vector] = None,
        archived: Optional[bool] = False,  # Delete
    ) -> None:
        data_ = parse_for_pg_creation(
            db_id=pg_id, data=data, icon=icon, cover_url=cover_url
        )  # the pg_id and db_id params are little bit confusing to understand but it works just fine
        data_.update(constants.ARCHIVE(archived))
        data_ = parse_into_json(data_)
        requests.patch(
            url=constants.UPDATE_PAGE_URL(pg_id),
            headers=constants.HEADERS(Update.TOKEN),
            data=data_,
        )

    # not currently in use for some bugs but left for feature updates
    def db(
        self,
        db_id: str,
        title: str,
        icon: Optional[str] = None,
        cover_url: Optional[str] = None,
        data: Vector = None,
        archived: Optional[bool] = False,  # Delete
    ) -> None:
        data_ = parse_for_updating_db(
            db_id=db_id, data=data, title=title, icon=icon, cover_url=cover_url
        )  # the pg_id and db_id params are little bit confusing to understand but it works just fine
        data_.update(constants.ARCHIVE(archived))
        data_ = parse_into_json(data_)
        res = requests.patch(
            url=constants.UPDATE_PAGE_URL(db_id),
            headers=constants.HEADERS(Update.TOKEN),
            data=data_,
        )



from typing import Optional

import requests

from NotionPy import constants
from NotionPy.utils import parse_into_dict, parse_into_json


class Query:
    """
    class that retrieves data from page or database with option to get it
    json or dict like data
    """

    TOKEN = None

    def __init__(self) -> None:
        pass

    def page(
        self,
        page_id: str,
        in_json: Optional[bool] = False,
        json_indent: Optional[int] = None,
        print_data: Optional[bool] = False,
    ):
        res = requests.post(
            url=constants.QUERY_PAGE_URL(page_id),
            headers=constants.HEADERS(Query.TOKEN),
        )
        data = (
            parse_into_json(res.json(), json_indent)
            if in_json is True
            else parse_into_dict(parse_into_json(res.json(), json_indent))
        )
        if print_data is True:
            print(data)

    def db(
        self,
        db_id,
        in_json: Optional[bool] = False,
        json_indent: Optional[int] = None,
        print_data: Optional[bool] = False,
    ):
        res = requests.post(
            url=constants.QUERY_DB_URL(db_id), headers=constants.HEADERS(Query.TOKEN)
        )
        data = (
            parse_into_json(res.json(), json_indent)
            if in_json is True
            else parse_into_dict(parse_into_json(res.json(), json_indent))
        )
        if print_data == True:
            print(data)



#        ________________________
#        |\                      \
#        | \ _____________________\
#        |  |                      |
#        |  |       |\     |       |
#        |  |       | \    |       |
#        |  |       |  \   |       |
#        |  |       |   \  |       |
#        \  |       |    \ |       |
#         \ |       |     \|       |
#          \|______________________|

__version__ = "0.0.2"

"""
################# notionpy library ####################


-> notionpy is a python library made for simplifying interaction with notion for developers 
to be able to integrate notion with your programme easily 

-> Basic usage :
        >>> Creating page \ database
                NotionClient.create.page\db
        >>> Querying page \ database
                NotionClient.query.page\db
        >>> Updating page 
                NotionClient.update.page

-> Supported properties :
        -title          -rich_text
        -select         -multi_select
        -relation       -people
        -checkbox       -url
        -files          -email
        -phone          -number

::copyright: (c) 2022 by Kareem Ebrahim.:

"""



HEADERS = lambda token: {
    "Authorization": "Bearer " + token,
    "Notion-Version": "2022-02-22",
    "Content-Type": "application/json",
}

CREATE_DB_URL = "https://api.notion.com/v1/databases"

CREATE_PAGE_URL = "https://api.notion.com/v1/pages"

QUERY_DB_URL = lambda dbid: f"https://api.notion.com/v1/databases/{dbid}/query"

QUERY_PAGE_URL = lambda pgid: f"https://api.notion.com/v1/pages/{pgid}"

UPDATE_PAGE_URL = lambda pgid: f"https://api.notion.com/v1/pages/{pgid}"

ICON = lambda icon: {"icon": {"type": "emoji", "emoji": icon}}

COVER = lambda cover_url: {
    "cover": {"type": "external", "external": {"url": cover_url}}
}

ARCHIVE = lambda state: {"archived": state}

DB_TITLE = lambda title: {
    "title": [{"type": "text", "text": {"content": title, "link": None}}]
}

CREATING_PAGE_TEMPLATE = lambda dbid, data=None: {
    "parent": {
        "database_id": dbid,
    },
    "properties": data,
}

CREATING_DATABASE_TEMPLATE = (
    lambda page_id, title=None, data=None: {
        "parent": {"type": "page_id", "page_id": page_id},
        "title": [{"type": "text", "text": {"content": title, "link": None}}],
        "properties": data,
    }
    if title is not None
    else {
        "parent": {"type": "page_id", "page_id": page_id},
        "properties": data,
    }
)

# not currently in use but for future updates
UPDATE_DB_TEMPLATE = (
    lambda db_id, data=None: {
        "parent": {
            "database_id": db_id,
        },
        "properties": data,
    }
    if data is not None
    else {
        "parent": {
            "database_id": db_id,
        },
    }
)

TITLE = lambda prop_name, content: {
    prop_name: {"title": [{"type": "text", "text": {"content": content}}]}
}

RICH_TEXT = lambda prop_name, content: {
    prop_name: {"rich_text": [{"type": "text", "text": {"content": content}}]}
}

SELECT = lambda prop_name, option_name: {prop_name: {"select": {"name": option_name}}}

MULTI_SELECT = lambda prop_name, select_options: {
    prop_name: {"multi_select": [{"name": option} for option in select_options]}
}

RELATION = lambda prop_name, releated_pg_id: {
    prop_name: {"relation": [{"id": releated_pg_id}]}
}

PEOPLE = lambda prop_name, person_id: {
    prop_name: {"people": [{"object": "user", "id": p_id} for p_id in person_id]}
}

CHECKBOX = lambda prop_name, status: {prop_name: {"checkbox": status}}

URL = lambda prop_name, content: {prop_name: {"url": content}}

FILES = lambda prop_name, url: {
    prop_name: {
        "files": [{"type": "external", "name": "content", "external": {"url": url}}]
    }
}

EMAIL = lambda prop_name, email: {prop_name: {"email": email}}

PHONE = lambda prop_name, phone: {prop_name: {"phone_number": phone}}

NUMBER = lambda prop_name, number: {prop_name: {"number": number}}



import pathlib
from setuptools import setup

HERE = pathlib.Path(__file__).parent

README = (HERE / "README.md").read_text()

setup(
    name="auto-py-notion",
    version="0.0.2",
    description="integrate notion api with your workflow",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/kareemmahlees/NotionPy",
    author="Kareem Ebrahim",
    author_email="kareemmahlees@gmail.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.10",
    ],
    packages=["NotionPy"],
    include_package_data=True,
    install_requires=["requests"],
    # entry_points={
    #     "console_scripts": [
    #         "realpython=reader.__main__:main",
    #     ]
    # },
)



from NotionPy.notionpy import NotionClient

# inst = NotionClient("Your integration token")
# inst.create.page(
#     database_id="the id of database of choice",
#     data=[  # List of tuples
#         ("Name", "title", "kareem"),
#         ("price", "number", 254),
#         ("to-do", "checkbox", False),
#     ],
#     # Optional
#     icon="🔥",
#     # Optional
#     cover="https://images.unsplash.com/photo-1523867574998-1a336b6ded04?ixlib=rb-1.2.1&ixid=MnwxMjA3fDB8MHxzZWFyY2h8Mnx8Y292ZXJ8ZW58MHx8MHx8&w=1000&q=80",
# )

# inst.create.db(
#     "cc39932a53da46be8472fdfc4c55966e",
#     "kareem's db",
#     [("Name", "title"), ("num", "number")],
#     "🌟",
# )

# secret_LYxhmvMw0RoQncukgmKNjLkPHduLBLgELz5HE9id5MW
#
# inst.create.page(
#     "04cd159d03a64f98bdcba4cb45014d30",
#     data=[("Name", "title", "kareem"), ("number", "phone", "01006007952")],
# )
# inst.update.page(
#     "d6ee44d9d4ef4a8a85cb7d0a1c69364d",
#     [("Name", "title", "ahmed"), ("number", "phone", "56748625452")],
#     # archived=True,
#     icon="🦴",
# )
# inst.update.db(
#     "04cd159d03a64f98bdcba4cb45014d30",
#     # title="Grocery list",
#     data=[("Name", "title", "ahmed")],
#     icon="📌",
# )
inst = NotionClient("lkasjdlkfj")



from typing import List, Tuple, Optional, Any


from NotionPy.create import Create
from NotionPy.query import Query
from NotionPy.update import Update


class NotionClient:
    """
    A class that acts like an entry point for other classes methods
    """

    def __init__(self, token: str) -> None:
        Create.TOKEN = token
        Query.TOKEN = token
        Update.TOKEN = token

    class create:
        def page(
            database_id: str,
            data: List[Tuple[str, str, Optional[Any]]],
            icon: Optional[str] = None,
            cover: Optional[str] = None,
        ) -> None:
            Create.page(Create, database_id, icon, cover, data)

        def db(
            page_id: str,
            title: str,
            data: List[Tuple[str, str]],
            icon: Optional[str] = None,
            cover: Optional[str] = None,
        ) -> None:
            Create.db(Create, page_id, title, icon, cover, data)

    class query:
        def page(
            page_id,
            in_json: Optional[bool] = False,
            json_indent: Optional[int] = None,
            print_data: Optional[bool] = False,
        ) -> None:
            Query.page(Query, page_id, in_json, json_indent, print_data)

        def db(
            db_id,
            in_json: Optional[bool] = False,
            json_indent: Optional[int] = None,
            print_data: Optional[bool] = False,
        ) -> None:
            Query.db(Query, db_id, in_json, json_indent, print_data)

    class update:
        def page(
            page_id,
            data: List[Tuple[str, str, Optional[Any]]],
            icon: Optional[str] = None,
            cover: Optional[str] = None,
            archived: Optional[bool] = False,
        ):
            Update.page(Update, page_id, icon, cover, data, archived)

        # not currently in use but for          future updates
        def db(
            page_id,
            title: Optional[str] = None,
            data: Optional[List[Tuple[str, str, Optional[Any]]]] = None,
            icon: Optional[str] = None,
            cover: Optional[str] = None,
            archived: Optional[bool] = False,
        ):
            Update.db(Update, page_id, title, icon, cover, data, archived)
